@@ -4,9 +4,9 @@
 static void misra_10_4(void)
 {
     // #10480
-    char buf[1] = {'f'};
+    const char buf[1] = {'f'};
     const char c = '0';
-    signed int x = buf[0] - c;
+    x = buf[0] - c;
 }
 
 static void misra_12_2(void) {
