@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2022 Cppcheck team.
+ * Copyright (C) 2007-2023 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,8 @@
 #include "checkstatistics.h"
 
 #include <QDebug>
+#include <QList>
+#include <QSet>
 
 CheckStatistics::CheckStatistics(QObject *parent)
     : QObject(parent)
