@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2022 Cppcheck team.
+ * Copyright (C) 2007-2023 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,7 +22,14 @@
 #include "codeeditstylecontrols.h"
 
 #include <QDialogButtonBox>
+#include <QFlags>
+#include <QFontMetrics>
 #include <QFormLayout>
+#include <QHBoxLayout>
+#include <QPushButton>
+#include <QStringList>
+#include <QVBoxLayout>
+#include <QtGlobal>
 
 class QWidget;
 
