@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2022 Cppcheck team.
+ * Copyright (C) 2007-2023 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,26 +20,16 @@
 #define STATSDIALOG_H
 
 #include <QDialog>
+#include <QObject>
+#include <QString>
 
 class ProjectFile;
 class CheckStatistics;
-class QObject;
 class QWidget;
 namespace Ui {
     class StatsDialog;
 }
 
-#ifdef HAVE_QCHART
-#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
-namespace QtCharts {
-#endif
-class QChartView;
-class QLineSeries;
-#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
-}
-#endif
-#endif
-
 /// @addtogroup GUI
 /// @{
 
@@ -81,15 +71,6 @@ public:
 private slots:
     void copyToClipboard();
     void pdfExport();
-#ifdef HAVE_QCHART
-#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
-    QChartView *createChart(const QString &statsFile, const QString &tool);
-    QLineSeries *numberOfReports(const QString &fileName, const QString &severity) const;
-#else
-    QtCharts::QChartView *createChart(const QString &statsFile, const QString &tool);
-    QtCharts::QLineSeries *numberOfReports(const QString &fileName, const QString &severity) const;
-#endif
-#endif
 private:
     Ui::StatsDialog *mUI;
     const CheckStatistics *mStatistics;
