@@ -18,9 +18,19 @@
 
 #include "testprojectfile.h"
 
+#include "importproject.h"
+#include "library.h"
+#include "platform.h"
 #include "projectfile.h"
 #include "settings.h"
 
+#include <set>
+#include <string>
+#include <utility>
+#include <vector>
+
+#include <QList>
+#include <QStringList>
 #include <QtTest>
 
 // Mock...
