@@ -24,6 +24,7 @@
 
 #include "astutils.h"
 #include "mathlib.h"
+#include "platform.h"
 #include "standards.h"
 #include "symboldatabase.h"
 #include "token.h"
@@ -32,6 +33,7 @@
 #include "vfvalue.h"
 
 #include <iomanip>
+#include <list>
 #include <sstream>
 #include <unordered_map>
 #include <vector>
@@ -249,12 +251,12 @@ void CheckFunctions::checkIgnoredReturnValue()
             else if (Token::Match(tok, "[(<]") && tok->link())
                 tok = tok->link();
 
-            if (tok->varId() || !Token::Match(tok, "%name% (") || tok->isKeyword())
+            if (tok->varId() || tok->isKeyword() || tok->isStandardType() || !Token::Match(tok, "%name% ("))
                 continue;
 
             const Token *parent = tok->next()->astParent();
             while (Token::Match(parent, "%cop%")) {
-                if (Token::Match(parent, "<<|>>") && !parent->astParent())
+                if (Token::Match(parent, "<<|>>|*") && !parent->astParent())
                     break;
                 parent = parent->astParent();
             }
@@ -267,7 +269,7 @@ void CheckFunctions::checkIgnoredReturnValue()
             }
 
             if ((!tok->function() || !Token::Match(tok->function()->retDef, "void %name%")) &&
-                !WRONG_DATA(!tok->next()->astOperand1(), tok)) {
+                tok->next()->astOperand1()) {
                 const Library::UseRetValType retvalTy = mSettings->library.getUseRetValType(tok);
                 const bool warn = (tok->function() && tok->function()->isAttributeNodiscard()) || // avoid duplicate warnings for resource-allocating functions
                                   (retvalTy == Library::UseRetValType::DEFAULT && mSettings->library.getAllocFuncInfo(tok) == nullptr);
@@ -393,11 +395,7 @@ static const Token *checkMissingReturnScope(const Token *tok, const Library &lib
         if (tok->str() == "goto" && !isForwardJump(tok))
             return nullptr;
         if (Token::Match(tok, "%name% (") && !library.isnotnoreturn(tok)) {
-            const Token *start = tok;
-            while (Token::Match(start->tokAt(-2), "%name% :: %name%"))
-                start = start->tokAt(-2);
-            if (Token::Match(start->previous(), "[;{}] %name% ::|("))
-                return nullptr;
+            return nullptr;
         }
         if (Token::Match(tok, "[;{}] %name% :"))
             return tok;
@@ -563,8 +561,8 @@ void CheckFunctions::memsetInvalid2ndParam()
 
             if (printWarning && secondParamTok->isNumber()) { // Check if the second parameter is a literal and is out of range
                 const long long int value = MathLib::toLongNumber(secondParamTok->str());
-                const long long sCharMin = mSettings->signedCharMin();
-                const long long uCharMax = mSettings->unsignedCharMax();
+                const long long sCharMin = mSettings->platform.signedCharMin();
+                const long long uCharMax = mSettings->platform.unsignedCharMax();
                 if (value < sCharMin || value > uCharMax)
                     memsetValueOutOfRangeError(secondParamTok, secondParamTok->str());
             }
@@ -624,6 +622,14 @@ void CheckFunctions::checkLibraryMatchFunctions()
         if (Token::simpleMatch(tok->astTop(), "throw"))
             continue;
 
+        if (Token::simpleMatch(tok->astParent(), ".")) {
+            const Token* contTok = tok->astParent()->astOperand1();
+            if (astContainerAction(contTok) != Library::Container::Action::NO_ACTION)
+                continue;
+            if (astContainerYield(contTok) != Library::Container::Yield::NO_YIELD)
+                continue;
+        }
+
         if (!mSettings->library.isNotLibraryFunction(tok))
             continue;
 
@@ -637,6 +643,9 @@ void CheckFunctions::checkLibraryMatchFunctions()
         if (mSettings->library.podtype(tok->expressionString()))
             continue;
 
+        if (mSettings->library.getTypeCheck("unusedvar", functionName) != Library::TypeCheck::def)
+            continue;
+
         const Token* start = tok;
         while (Token::Match(start->tokAt(-2), "%name% ::"))
             start = start->tokAt(-2);
