@@ -192,9 +192,9 @@ private:
     void string_c_strThrowError(const Token* tok);
     void string_c_strError(const Token* tok);
     void string_c_strReturn(const Token* tok);
-    void string_c_strParam(const Token* tok, nonneg int number);
-    void string_c_strConstructor(const Token* tok);
-    void string_c_strAssignment(const Token* tok);
+    void string_c_strParam(const Token* tok, nonneg int number, const std::string& argtype = "std::string");
+    void string_c_strConstructor(const Token* tok, const std::string& argtype = "std::string");
+    void string_c_strAssignment(const Token* tok, const std::string& argtype = "std::string");
     void string_c_strConcat(const Token* tok);
     void string_c_strStream(const Token* tok);
 
