@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2022 Cppcheck team.
+ * Copyright (C) 2007-2023 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,27 +20,46 @@
 
 #ifndef _WIN32
 #include <unistd.h>
-#endif
 #include <cstddef>
 #include <sstream> // IWYU pragma: keep
+#include <iostream>
+#endif
 
-#ifdef _WIN32
-std::ostream& operator<<(std::ostream& os, const Color& /*c*/)
+bool gDisableColors = false;
+
+#ifndef _WIN32
+static bool isStreamATty(const std::ostream & os)
 {
-#else
+    static const bool stdout_tty = isatty(STDOUT_FILENO);
+    static const bool stderr_tty = isatty(STDERR_FILENO);
+    if (&os == &std::cout)
+        return stdout_tty;
+    if (&os == &std::cerr)
+        return stderr_tty;
+    return (stdout_tty && stderr_tty);
+}
+#endif
+
 std::ostream& operator<<(std::ostream & os, const Color& c)
 {
-    static const bool use_color = isatty(STDOUT_FILENO);
-    if (use_color)
+#ifndef _WIN32
+    if (!gDisableColors && isStreamATty(os))
         return os << "\033[" << static_cast<std::size_t>(c) << "m";
+#else
+    (void)c;
 #endif
     return os;
 }
 
 std::string toString(const Color& c)
 {
+#ifndef _WIN32
     std::stringstream ss;
     ss << c;
     return ss.str();
+#else
+    (void)c;
+    return "";
+#endif
 }
 
