@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2021 Cppcheck team.
+ * Copyright (C) 2007-2023 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,14 +32,12 @@ enum class Color {
     FgGreen    = 32,
     FgBlue     = 34,
     FgMagenta  = 35,
-    FgDefault  = 39,
-    BgRed      = 41,
-    BgGreen    = 42,
-    BgBlue     = 44,
-    BgDefault  = 49
+    FgDefault  = 39
 };
 CPPCHECKLIB std::ostream& operator<<(std::ostream& os, const Color& c);
 
-std::string toString(const Color& c);
+CPPCHECKLIB std::string toString(const Color& c);
+
+extern CPPCHECKLIB bool gDisableColors; // for testing
 
 #endif
