@@ -148,7 +148,8 @@ struct Interval {
                 result.setMinValue(minValue->intvalue + 1, minValue);
             if (minValue->isPossible() && minValue->bound == ValueFlow::Value::Bound::Lower)
                 result.setMinValue(minValue->intvalue, minValue);
-            if (minValue->isKnown())
+            if (!minValue->isImpossible() && (minValue->bound == ValueFlow::Value::Bound::Point || minValue->isKnown()) &&
+                std::count_if(values.begin(), values.end(), predicate) == 1)
                 return Interval::fromInt(minValue->intvalue, minValue);
         }
         const ValueFlow::Value* maxValue = getCompareValue(values, predicate, std::greater<MathLib::bigint>{});
@@ -226,8 +227,7 @@ struct Interval {
         if (!eq.empty()) {
             if (eq.front() == 0)
                 return {1, -1};
-            else
-                return {0};
+            return {0};
         }
         if (diff.isGreaterThan(-1, ref))
             return {0, 1};
@@ -269,6 +269,12 @@ static void addToErrorPath(ValueFlow::Value& value, const std::vector<const Valu
                      [&](const ErrorPathItem& e) {
             return locations.insert(e.first).second;
         });
+        std::copy_if(ref->debugPath.cbegin(),
+                     ref->debugPath.cend(),
+                     std::back_inserter(value.debugPath),
+                     [&](const ErrorPathItem& e) {
+            return locations.insert(e.first).second;
+        });
     }
 }
 
