@@ -101,16 +101,16 @@ std::string Suppressions::parseXmlFile(const char *filename)
             else if (std::strcmp(e2->Name(), "fileName") == 0)
                 s.fileName = text;
             else if (std::strcmp(e2->Name(), "lineNumber") == 0)
-                s.lineNumber = std::atoi(text);
+                s.lineNumber = strToInt<int>(text);
             else if (std::strcmp(e2->Name(), "symbolName") == 0)
                 s.symbolName = text;
             else if (*text && std::strcmp(e2->Name(), "hash") == 0)
-                std::istringstream(text) >> s.hash;
+                s.hash = strToInt<std::size_t>(text);
             else
                 return "Unknown suppression element \"" + std::string(e2->Name()) + "\", expected id/fileName/lineNumber/symbolName/hash";
         }
 
-        const std::string err = addSuppression(s);
+        const std::string err = addSuppression(std::move(s));
         if (!err.empty())
             return err;
     }
@@ -123,8 +123,8 @@ std::vector<Suppressions::Suppression> Suppressions::parseMultiSuppressComment(c
     std::vector<Suppression> suppressions;
 
     // If this function is called we assume that comment starts with "cppcheck-suppress[".
-    const std::string::size_type start_position = comment.find("[");
-    const std::string::size_type end_position = comment.find("]", start_position);
+    const std::string::size_type start_position = comment.find('[');
+    const std::string::size_type end_position = comment.find(']', start_position);
     if (end_position == std::string::npos) {
         if (errorMessage && errorMessage->empty())
             *errorMessage = "Bad multi suppression '" + comment + "'. legal format is cppcheck-suppress[errorId, errorId symbolName=arr, ...]";
@@ -134,7 +134,7 @@ std::vector<Suppressions::Suppression> Suppressions::parseMultiSuppressComment(c
     // parse all suppressions
     for (std::string::size_type pos = start_position; pos < end_position;) {
         const std::string::size_type pos1 = pos + 1;
-        pos = comment.find(",", pos1);
+        pos = comment.find(',', pos1);
         const std::string::size_type pos2 = (pos < end_position) ? pos : end_position;
         if (pos1 == pos2)
             continue;
@@ -179,7 +179,7 @@ std::string Suppressions::addSuppressionLine(const std::string &line)
     Suppressions::Suppression suppression;
 
     // Strip any end of line comments
-    std::string::size_type endpos = std::min(line.find("#"), line.find("//"));
+    std::string::size_type endpos = std::min(line.find('#'), line.find("//"));
     if (endpos != std::string::npos) {
         while (endpos > 0 && std::isspace(line[endpos-1])) {
             endpos--;
@@ -218,10 +218,10 @@ std::string Suppressions::addSuppressionLine(const std::string &line)
 
     suppression.fileName = Path::simplifyPath(suppression.fileName);
 
-    return addSuppression(suppression);
+    return addSuppression(std::move(suppression));
 }
 
-std::string Suppressions::addSuppression(const Suppressions::Suppression &suppression)
+std::string Suppressions::addSuppression(Suppressions::Suppression suppression)
 {
     // Check if suppression is already in list
     auto foundSuppression = std::find_if(mSuppressions.begin(), mSuppressions.end(),
@@ -253,15 +253,15 @@ std::string Suppressions::addSuppression(const Suppressions::Suppression &suppre
     if (!isValidGlobPattern(suppression.fileName))
         return "Failed to add suppression. Invalid glob pattern '" + suppression.fileName + "'.";
 
-    mSuppressions.push_back(suppression);
+    mSuppressions.push_back(std::move(suppression));
 
     return "";
 }
 
-std::string Suppressions::addSuppressions(const std::list<Suppression> &suppressions)
+std::string Suppressions::addSuppressions(std::list<Suppression> suppressions)
 {
-    for (const auto &newSuppression : suppressions) {
-        auto errmsg = addSuppression(newSuppression);
+    for (auto &newSuppression : suppressions) {
+        auto errmsg = addSuppression(std::move(newSuppression));
         if (!errmsg.empty())
             return errmsg;
     }
@@ -367,10 +367,12 @@ std::string Suppressions::Suppression::getText() const
     return ret;
 }
 
-bool Suppressions::isSuppressed(const Suppressions::ErrorMessage &errmsg)
+bool Suppressions::isSuppressed(const Suppressions::ErrorMessage &errmsg, bool global)
 {
     const bool unmatchedSuppression(errmsg.errorId == "unmatchedSuppression");
     for (Suppression &s : mSuppressions) {
+        if (!global && !s.isLocal())
+            continue;
         if (unmatchedSuppression && s.errorId != errmsg.errorId)
             continue;
         if (s.isMatch(errmsg))
@@ -379,18 +381,11 @@ bool Suppressions::isSuppressed(const Suppressions::ErrorMessage &errmsg)
     return false;
 }
 
-bool Suppressions::isSuppressedLocal(const Suppressions::ErrorMessage &errmsg)
+bool Suppressions::isSuppressed(const ::ErrorMessage &errmsg)
 {
-    const bool unmatchedSuppression(errmsg.errorId == "unmatchedSuppression");
-    for (Suppression &s : mSuppressions) {
-        if (!s.isLocal())
-            continue;
-        if (unmatchedSuppression && s.errorId != errmsg.errorId)
-            continue;
-        if (s.isMatch(errmsg))
-            return true;
-    }
-    return false;
+    if (mSuppressions.empty())
+        return false;
+    return isSuppressed(errmsg.toSuppressionsErrorMessage());
 }
 
 void Suppressions::dump(std::ostream & out) const
