@@ -97,6 +97,7 @@ static const std::unordered_set<std::string> controlFlowKeywords = {
     "return"
 };
 
+// TODO: replace with Keywords::getX()?
 // Another list of keywords
 static const std::unordered_set<std::string> baseKeywords = {
     "asm",
@@ -231,11 +232,9 @@ bool Token::isUpperCaseName() const
 {
     if (!isName())
         return false;
-    for (const char i : mStr) {
-        if (std::islower(i))
-            return false;
-    }
-    return true;
+    return std::none_of(mStr.begin(), mStr.end(), [](char c) {
+        return std::islower(c);
+    });
 }
 
 void Token::concatStr(std::string const& b)
@@ -436,14 +435,6 @@ static int multiComparePercent(const Token *tok, const char*& haystack, nonneg i
     ++haystack;
     // Compare only the first character of the string for optimization reasons
     switch (haystack[0]) {
-    case '\0':
-    case ' ':
-    case '|':
-        //simple '%' character
-        haystack += 1;
-        if (tok->isArithmeticalOp() && tok->str() == "%")
-            return 1;
-        break;
     case 'v':
         if (haystack[3] == '%') { // %var%
             haystack += 4;
@@ -474,11 +465,11 @@ static int multiComparePercent(const Token *tok, const char*& haystack, nonneg i
         if (haystack[3] == '%') { // %any%
             haystack += 4;
             return 1;
-        } else { // %assign%
-            haystack += 7;
-            if (tok->isAssignmentOp())
-                return 1;
         }
+        // %assign%
+        haystack += 7;
+        if (tok->isAssignmentOp())
+            return 1;
     }
     break;
     case 'n':
@@ -655,9 +646,9 @@ bool Token::simpleMatch(const Token *tok, const char pattern[], size_t pattern_l
 bool Token::firstWordEquals(const char *str, const char *word)
 {
     for (;;) {
-        if (*str != *word) {
+        if (*str != *word)
             return (*str == ' ' && *word == 0);
-        } else if (*str == 0)
+        if (*str == 0)
             break;
 
         ++str;
@@ -845,12 +836,12 @@ void Token::move(Token *srcStart, Token *srcEnd, Token *newLocation)
         tok->mImpl->mProgressValue = newLocation->mImpl->mProgressValue;
 }
 
-Token* Token::nextArgument() const
+const Token* Token::nextArgument() const
 {
     for (const Token* tok = this; tok; tok = tok->next()) {
         if (tok->str() == ",")
             return tok->next();
-        else if (tok->link() && Token::Match(tok, "(|{|[|<"))
+        if (tok->link() && Token::Match(tok, "(|{|[|<"))
             tok = tok->link();
         else if (Token::Match(tok, ")|;"))
             return nullptr;
@@ -858,12 +849,12 @@ Token* Token::nextArgument() const
     return nullptr;
 }
 
-Token* Token::nextArgumentBeforeCreateLinks2() const
+const Token* Token::nextArgumentBeforeCreateLinks2() const
 {
     for (const Token* tok = this; tok; tok = tok->next()) {
         if (tok->str() == ",")
             return tok->next();
-        else if (tok->link() && Token::Match(tok, "(|{|["))
+        if (tok->link() && Token::Match(tok, "(|{|["))
             tok = tok->link();
         else if (tok->str() == "<") {
             const Token* temp = tok->findClosingBracket();
@@ -875,12 +866,12 @@ Token* Token::nextArgumentBeforeCreateLinks2() const
     return nullptr;
 }
 
-Token* Token::nextTemplateArgument() const
+const Token* Token::nextTemplateArgument() const
 {
     for (const Token* tok = this; tok; tok = tok->next()) {
         if (tok->str() == ",")
             return tok->next();
-        else if (tok->link() && Token::Match(tok, "(|{|[|<"))
+        if (tok->link() && Token::Match(tok, "(|{|[|<"))
             tok = tok->link();
         else if (Token::Match(tok, ">|;"))
             return nullptr;
@@ -1468,7 +1459,7 @@ std::pair<const Token *, const Token *> Token::findExpressionStartEndTokens() co
             }
         }
         if (Token::Match(end,"(|[|{") &&
-            !(Token::Match(end, "( %type%") && !end->astOperand2())) {
+            !(Token::Match(end, "( ::| %type%") && !end->astOperand2())) {
             end = end->link();
             break;
         }
@@ -1953,16 +1944,15 @@ static bool removeContradiction(std::list<ValueFlow::Value>& values)
                 if (removey)
                     values.remove(y);
                 return true;
-            } else {
-                result = removex || removey;
-                bool bail = false;
-                if (removex && removePointValue(values, x))
-                    bail = true;
-                if (removey && removePointValue(values, y))
-                    bail = true;
-                if (bail)
-                    return true;
             }
+            result = removex || removey;
+            bool bail = false;
+            if (removex && removePointValue(values, x))
+                bail = true;
+            if (removey && removePointValue(values, y))
+                bail = true;
+            if (bail)
+                return true;
         }
     }
     return result;
@@ -2161,7 +2151,8 @@ bool Token::addValue(const ValueFlow::Value &value)
         ValueFlow::Value v(value);
         if (v.varId == 0)
             v.varId = mImpl->mVarId;
-        mImpl->mValues = new std::list<ValueFlow::Value>(1, v);
+        mImpl->mValues = new std::list<ValueFlow::Value>;
+        mImpl->mValues->push_back(std::move(v));
     }
 
     removeContradictions(*mImpl->mValues);
@@ -2211,13 +2202,13 @@ const ::Type* Token::typeOf(const Token* tok, const Token** typeTok)
     if (typeTok != nullptr)
         *typeTok = tok;
     const Token* lhsVarTok{};
-    if (tok->type()) {
+    if (tok->type())
         return tok->type();
-    } else if (tok->variable()) {
+    if (tok->variable())
         return tok->variable()->type();
-    } else if (tok->function()) {
+    if (tok->function())
         return tok->function()->retType;
-    } else if (Token::simpleMatch(tok, "return")) {
+    if (Token::simpleMatch(tok, "return")) {
         const Scope *scope = tok->scope();
         if (!scope)
             return nullptr;
@@ -2225,15 +2216,16 @@ const ::Type* Token::typeOf(const Token* tok, const Token** typeTok)
         if (!function)
             return nullptr;
         return function->retType;
-    } else if (Token::Match(tok->previous(), "%type%|= (|{")) {
+    }
+    if (Token::Match(tok->previous(), "%type%|= (|{"))
         return typeOf(tok->previous(), typeTok);
-    } else if (Token::simpleMatch(tok, "=") && (lhsVarTok = getLHSVariableToken(tok)) != tok->next()) {
+    if (Token::simpleMatch(tok, "=") && (lhsVarTok = getLHSVariableToken(tok)) != tok->next())
         return Token::typeOf(lhsVarTok, typeTok);
-    } else if (Token::simpleMatch(tok, ".")) {
+    if (Token::simpleMatch(tok, "."))
         return Token::typeOf(tok->astOperand2(), typeTok);
-    } else if (Token::simpleMatch(tok, "[")) {
+    if (Token::simpleMatch(tok, "["))
         return Token::typeOf(tok->astOperand1(), typeTok);
-    } else if (Token::simpleMatch(tok, "{")) {
+    if (Token::simpleMatch(tok, "{")) {
         int argnr;
         const Token* ftok = getTokenArgumentFunction(tok, argnr);
         if (argnr < 0)
@@ -2259,12 +2251,22 @@ std::pair<const Token*, const Token*> Token::typeDecl(const Token* tok, bool poi
 {
     if (!tok)
         return {};
-    else if (tok->type()) {
+    if (tok->type())
         return {tok, tok->next()};
-    } else if (tok->variable()) {
+    if (tok->variable()) {
         const Variable *var = tok->variable();
         if (!var->typeStartToken() || !var->typeEndToken())
             return {};
+        if (pointedToType && astIsSmartPointer(var->nameToken())) {
+            const ValueType* vt = var->valueType();
+            if (vt && vt->smartPointerTypeToken)
+                return { vt->smartPointerTypeToken, vt->smartPointerTypeToken->linkAt(-1) };
+        }
+        if (pointedToType && astIsIterator(var->nameToken())) {
+            const ValueType* vt = var->valueType();
+            if (vt && vt->containerTypeToken)
+                return { vt->containerTypeToken, vt->containerTypeToken->linkAt(-1) };
+        }
         std::pair<const Token*, const Token*> result;
         if (Token::simpleMatch(var->typeStartToken(), "auto")) {
             const Token * tok2 = var->declEndToken();
@@ -2272,9 +2274,24 @@ std::pair<const Token*, const Token*> Token::typeDecl(const Token* tok, bool poi
                 tok2 = tok2->tokAt(2);
             if (Token::simpleMatch(tok2, "=") && Token::Match(tok2->astOperand2(), "!!=") && tok != tok2->astOperand2()) {
                 tok2 = tok2->astOperand2();
-                std::pair<const Token*, const Token*> r = typeDecl(tok2);
+
+                if (Token::simpleMatch(tok2, "[") && tok2->astOperand1()) {
+                    const ValueType* vt = tok2->astOperand1()->valueType();
+                    if (vt && vt->containerTypeToken)
+                        return { vt->containerTypeToken, vt->containerTypeToken->linkAt(-1) };
+                }
+
+                const Token* varTok = tok2; // try to find a variable
+                if (Token::Match(varTok, ":: %name%"))
+                    varTok = varTok->next();
+                while (Token::Match(varTok, "%name% ::"))
+                    varTok = varTok->tokAt(2);
+                if (Token::simpleMatch(varTok, "(") && Token::simpleMatch(varTok->astOperand1(), "."))
+                    varTok = varTok->astOperand1()->astOperand1();
+                std::pair<const Token*, const Token*> r = typeDecl(varTok);
                 if (r.first)
                     return r;
+
                 if (pointedToType && tok2->astOperand1() && Token::simpleMatch(tok2, "new")) {
                     if (Token::simpleMatch(tok2->astOperand1(), "("))
                         return { tok2->next(), tok2->astOperand1() };
@@ -2300,21 +2317,12 @@ std::pair<const Token*, const Token*> Token::typeDecl(const Token* tok, bool poi
                 if (vt && vt->containerTypeToken)
                     return { vt->containerTypeToken, vt->containerTypeToken->linkAt(-1) };
             }
-            if (pointedToType && astIsSmartPointer(var->nameToken())) {
-                const ValueType* vt = var->valueType();
-                if (vt && vt->smartPointerTypeToken)
-                    return { vt->smartPointerTypeToken, vt->smartPointerTypeToken->linkAt(-1) };
-            }
-            if (pointedToType && astIsIterator(var->nameToken())) {
-                const ValueType* vt = var->valueType();
-                if (vt && vt->containerTypeToken)
-                    return { vt->containerTypeToken, vt->containerTypeToken->linkAt(-1) };
-            }
-            if (result.first)
-                return result;
         }
+        if (result.first)
+            return result;
         return {var->typeStartToken(), var->typeEndToken()->next()};
-    } else if (Token::simpleMatch(tok, "return")) {
+    }
+    if (Token::simpleMatch(tok, "return")) {
         const Scope* scope = tok->scope();
         if (!scope)
             return {};
@@ -2322,19 +2330,20 @@ std::pair<const Token*, const Token*> Token::typeDecl(const Token* tok, bool poi
         if (!function)
             return {};
         return { function->retDef, function->returnDefEnd() };
-    } else if (tok->previous() && tok->previous()->function()) {
+    }
+    if (tok->previous() && tok->previous()->function()) {
         const Function *function = tok->previous()->function();
         return {function->retDef, function->returnDefEnd()};
-    } else if (Token::simpleMatch(tok, "=")) {
-        return Token::typeDecl(tok->astOperand1());
-    } else if (Token::simpleMatch(tok, ".")) {
-        return Token::typeDecl(tok->astOperand2());
-    } else {
-        const ::Type * t = typeOf(tok);
-        if (!t || !t->classDef)
-            return {};
-        return {t->classDef->next(), t->classDef->tokAt(2)};
     }
+    if (Token::simpleMatch(tok, "="))
+        return Token::typeDecl(tok->astOperand1());
+    if (Token::simpleMatch(tok, "."))
+        return Token::typeDecl(tok->astOperand2());
+
+    const ::Type * t = typeOf(tok);
+    if (!t || !t->classDef)
+        return {};
+    return {t->classDef->next(), t->classDef->tokAt(2)};
 }
 std::string Token::typeStr(const Token* tok)
 {
@@ -2489,13 +2498,13 @@ void TokenImpl::setCppcheckAttribute(TokenImpl::CppcheckAttributes::Type type, M
     }
 }
 
-bool TokenImpl::getCppcheckAttribute(TokenImpl::CppcheckAttributes::Type type, MathLib::bigint *value) const
+bool TokenImpl::getCppcheckAttribute(TokenImpl::CppcheckAttributes::Type type, MathLib::bigint &value) const
 {
     struct CppcheckAttributes *attr = mCppcheckAttributes;
     while (attr && attr->type != type)
         attr = attr->next;
     if (attr)
-        *value = attr->value;
+        value = attr->value;
     return attr != nullptr;
 }
 
@@ -2511,10 +2520,6 @@ Token* findTypeEnd(Token* tok)
     return tok;
 }
 
-const Token* findTypeEnd(const Token* tok) {
-    return findTypeEnd(const_cast<Token*>(tok));
-}
-
 Token* findLambdaEndScope(Token* tok)
 {
     if (!Token::simpleMatch(tok, "["))
