@@ -27,7 +27,6 @@
 #include <cstddef>
 #include <iosfwd>
 #include <string>
-#include <unordered_set>
 #include <vector>
 
 class Settings;
@@ -51,10 +50,6 @@ public:
         mSettings = settings;
     }
 
-    const Settings *getSettings() const {
-        return mSettings;
-    }
-
     /** @return the source file path. e.g. "file.cpp" */
     const std::string& getSourceFilePath() const;
 
@@ -212,8 +207,6 @@ private:
     /** settings */
     const Settings* mSettings;
 
-    std::unordered_set<std::string> mKeywords;
-
     /** File is known to be C/C++ code */
     bool mIsC;
     bool mIsCpp;
@@ -222,6 +215,7 @@ private:
 /// @}
 
 const Token* isLambdaCaptureList(const Token* tok);
+const Token* findLambdaEndTokenWithoutAST(const Token* tok);
 
 //---------------------------------------------------------------------------
 #endif // tokenlistH
