@@ -4,12 +4,13 @@
 #define CPPCHECK_MAJOR 2
 #define CPPCHECK_MINOR 11
 #define CPPCHECK_DEVMINOR 11
+#define CPPCHECK_PATCH 1
 
 #define STRINGIFY(x) STRING(x)
 #define STRING(VER) #VER
 #if CPPCHECK_MINOR == CPPCHECK_DEVMINOR
-#define CPPCHECK_VERSION_STRING STRINGIFY(CPPCHECK_MAJOR) "." STRINGIFY(CPPCHECK_DEVMINOR)
-#define CPPCHECK_VERSION CPPCHECK_MAJOR,CPPCHECK_MINOR,0,0
+#define CPPCHECK_VERSION_STRING STRINGIFY(CPPCHECK_MAJOR) "." STRINGIFY(CPPCHECK_DEVMINOR) "." STRINGIFY(CPPCHECK_PATCH)
+#define CPPCHECK_VERSION CPPCHECK_MAJOR,CPPCHECK_MINOR,CPPCHECK_PATCH,0
 #else
 #define CPPCHECK_VERSION_STRING STRINGIFY(CPPCHECK_MAJOR) "." STRINGIFY(CPPCHECK_DEVMINOR) " dev"
 #define CPPCHECK_VERSION CPPCHECK_MAJOR,CPPCHECK_MINOR,99,0
