@@ -23,6 +23,7 @@
 #include "fixture.h"
 #include "tokenize.h"
 
+#include <list>
 #include <sstream> // IWYU pragma: keep
 
 
@@ -31,7 +32,7 @@ public:
     TestAssert() : TestFixture("TestAssert") {}
 
 private:
-    Settings settings;
+    const Settings settings = settingsBuilder().severity(Severity::warning).build();
 
 #define check(...) check_(__FILE__, __LINE__, __VA_ARGS__)
     void check_(const char* file, int line, const char code[], const char *filename = "test.cpp") {
@@ -44,13 +45,10 @@ private:
         ASSERT_LOC(tokenizer.tokenize(istr, filename), file, line);
 
         // Check..
-        CheckAssert checkAssert;
-        checkAssert.runChecks(&tokenizer, &settings, this);
+        runChecks<CheckAssert>(&tokenizer, &settings, this);
     }
 
     void run() override {
-        settings.severity.enable(Severity::warning);
-
         TEST_CASE(assignmentInAssert);
         TEST_CASE(functionCallInAssert);
         TEST_CASE(memberFunctionCallInAssert);
