@@ -23,6 +23,7 @@
 #include "fixture.h"
 #include "tokenize.h"
 
+#include <list>
 #include <sstream> // IWYU pragma: keep
 
 class TestBool : public TestFixture {
@@ -30,13 +31,9 @@ public:
     TestBool() : TestFixture("TestBool") {}
 
 private:
-    Settings settings;
+    const Settings settings = settingsBuilder().severity(Severity::style).severity(Severity::warning).certainty(Certainty::inconclusive).build();
 
     void run() override {
-        settings.severity.enable(Severity::style);
-        settings.severity.enable(Severity::warning);
-        settings.certainty.enable(Certainty::inconclusive);
-
         TEST_CASE(bitwiseOnBoolean);      // if (bool & bool)
         TEST_CASE(incrementBoolean);
         TEST_CASE(assignBoolToPointer);
@@ -91,8 +88,7 @@ private:
         ASSERT_LOC(tokenizer.tokenize(istr, filename), file, line);
 
         // Check...
-        CheckBool checkBool(&tokenizer, &settings, this);
-        checkBool.runChecks(&tokenizer, &settings, this);
+        runChecks<CheckBool>(&tokenizer, &settings, this);
     }
 
 
