@@ -22,6 +22,7 @@
 #include "fixture.h"
 #include "tokenize.h"
 
+#include <list>
 #include <map>
 #include <sstream> // IWYU pragma: keep
 #include <string>
@@ -35,13 +36,9 @@ public:
     TestSizeof() : TestFixture("TestSizeof") {}
 
 private:
-    Settings settings;
+    const Settings settings = settingsBuilder().severity(Severity::warning).severity(Severity::portability).certainty(Certainty::inconclusive).library("std.cfg").build();
 
     void run() override {
-        settings.severity.enable(Severity::warning);
-        settings.severity.enable(Severity::portability);
-        settings.certainty.enable(Certainty::inconclusive);
-
         TEST_CASE(sizeofsizeof);
         TEST_CASE(sizeofCalculation);
         TEST_CASE(sizeofFunction);
@@ -66,8 +63,7 @@ private:
         ASSERT_LOC(tokenizer.tokenize(istr, "test.cpp"), file, line);
 
         // Check...
-        CheckSizeof checkSizeof(&tokenizer, &settings, this);
-        checkSizeof.runChecks(&tokenizer, &settings, this);
+        runChecks<CheckSizeof>(&tokenizer, &settings, this);
     }
 
     void checkP(const char code[]) {
@@ -90,8 +86,7 @@ private:
         tokenizer.simplifyTokens1("");
 
         // Check...
-        CheckSizeof checkSizeof(&tokenizer, &settings, this);
-        checkSizeof.runChecks(&tokenizer, &settings, this);
+        runChecks<CheckSizeof>(&tokenizer, &settings, this);
     }
 
     void sizeofsizeof() {
