@@ -18,6 +18,7 @@
 
 #include "settings.h"
 #include "fixture.h"
+#include "platform.h"
 #include "token.h"
 #include "tokenlist.h"
 
@@ -29,7 +30,7 @@ public:
     TestTokenList() : TestFixture("TestTokenList") {}
 
 private:
-    Settings settings;
+    const Settings settings;
 
     void run() override {
         TEST_CASE(testaddtoken1);
@@ -39,17 +40,18 @@ private:
     }
 
     // inspired by #5895
-    void testaddtoken1() {
+    void testaddtoken1() const {
         const std::string code = "0x89504e470d0a1a0a";
         TokenList tokenlist(&settings);
         tokenlist.addtoken(code, 1, 1, false);
         ASSERT_EQUALS("0x89504e470d0a1a0a", tokenlist.front()->str());
     }
 
-    void testaddtoken2() {
+    void testaddtoken2() const {
         const std::string code = "0xF0000000";
-        settings.int_bit = 32;
-        TokenList tokenlist(&settings);
+        Settings settings1;
+        settings1.platform.int_bit = 32;
+        TokenList tokenlist(&settings1);
         tokenlist.addtoken(code, 1, 1, false);
         ASSERT_EQUALS("0xF0000000", tokenlist.front()->str());
     }
@@ -67,7 +69,7 @@ private:
         ASSERT(Token::simpleMatch(tokenlist.front(), "a + + 1 ; 1 + + b ;"));
     }
 
-    void isKeyword() {
+    void isKeyword() const {
 
         const char code[] = "for a int delete true";
 
