@@ -24,6 +24,7 @@
 #include "fixture.h"
 #include "tokenize.h"
 
+#include <list>
 #include <sstream> // IWYU pragma: keep
 #include <string>
 
@@ -44,44 +45,37 @@ private:
     }
 
 #define check(...) check_(__FILE__, __LINE__, __VA_ARGS__)
-    void check_(const char* file, int line, const char code[], Settings* settings = nullptr, const char filename[] = "test.cpp", const std::string& standard = "c++11") {
+    void check_(const char* file, int line, const char code[], const Settings& settings, const char filename[] = "test.cpp", Standards::cppstd_t standard = Standards::cppstd_t::CPP11) {
         // Clear the error buffer..
         errout.str("");
 
-        if (!settings) {
-            static Settings _settings;
-            settings = &_settings;
-        }
-        settings->severity.enable(Severity::warning);
-        settings->severity.enable(Severity::portability);
-        settings->standards.setCPP(standard);
+        const Settings settings1 = settingsBuilder(settings).severity(Severity::warning).severity(Severity::portability).cpp(standard).build();
 
         // Tokenize..
-        Tokenizer tokenizer(settings, this);
+        Tokenizer tokenizer(&settings1, this);
         std::istringstream istr(code);
         ASSERT_LOC(tokenizer.tokenize(istr, filename), file, line);
 
         // Check..
-        CheckType checkType(&tokenizer, settings, this);
-        checkType.runChecks(&tokenizer, settings, this);
+        runChecks<CheckType>(&tokenizer, &settings1, this);
     }
 
     void checkTooBigShift_Unix32() {
-        Settings settings;
-        settings.platform(Settings::Unix32);
+        const Settings settings0;
+        const Settings settings = settingsBuilder().platform(cppcheck::Platform::Type::Unix32).build();
 
         // unsigned types getting promoted to int sizeof(int) = 4 bytes
         // and unsigned types having already a size of 4 bytes
         {
             const std::string types[] = {"unsigned char", /*[unsigned]*/ "char", "bool", "unsigned short", "unsigned int", "unsigned long"};
             for (const std::string& type : types) {
-                check((type + " f(" + type +" x) { return x << 31; }").c_str(), &settings);
+                check((type + " f(" + type +" x) { return x << 31; }").c_str(), settings);
                 ASSERT_EQUALS("", errout.str());
-                check((type + " f(" + type +" x) { return x << 33; }").c_str(), &settings);
+                check((type + " f(" + type +" x) { return x << 33; }").c_str(), settings);
                 ASSERT_EQUALS("[test.cpp:1]: (error) Shifting 32-bit value by 33 bits is undefined behaviour\n", errout.str());
-                check((type + " f(int x) { return (x = (" + type + ")x << 32); }").c_str(), &settings);
+                check((type + " f(int x) { return (x = (" + type + ")x << 32); }").c_str(), settings);
                 ASSERT_EQUALS("[test.cpp:1]: (error) Shifting 32-bit value by 32 bits is undefined behaviour\n", errout.str());
-                check((type + " foo(" + type + " x) { return x << 31; }").c_str(), &settings);
+                check((type + " foo(" + type + " x) { return x << 31; }").c_str(), settings);
                 ASSERT_EQUALS("", errout.str());
             }
         }
@@ -91,68 +85,68 @@ private:
             const std::string types[] = {"signed char", "signed short", /*[signed]*/ "short", "wchar_t", /*[signed]*/ "int", "signed int", /*[signed]*/ "long", "signed long"};
             for (const std::string& type : types) {
                 // c++11
-                check((type + " f(" + type +" x) { return x << 33; }").c_str(), &settings);
+                check((type + " f(" + type +" x) { return x << 33; }").c_str(), settings);
                 ASSERT_EQUALS("[test.cpp:1]: (error) Shifting 32-bit value by 33 bits is undefined behaviour\n", errout.str());
-                check((type + " f(int x) { return (x = (" + type + ")x << 32); }").c_str(), &settings);
+                check((type + " f(int x) { return (x = (" + type + ")x << 32); }").c_str(), settings);
                 ASSERT_EQUALS("[test.cpp:1]: (error) Shifting 32-bit value by 32 bits is undefined behaviour\n", errout.str());
-                check((type + " foo(" + type + " x) { return x << 31; }").c_str(), &settings);
+                check((type + " foo(" + type + " x) { return x << 31; }").c_str(), settings);
                 ASSERT_EQUALS("[test.cpp:1]: (error) Shifting signed 32-bit value by 31 bits is undefined behaviour\n", errout.str());
-                check((type + " foo(" + type + " x) { return x << 30; }").c_str(), &settings);
+                check((type + " foo(" + type + " x) { return x << 30; }").c_str(), settings);
                 ASSERT_EQUALS("", errout.str());
 
                 // c++14
-                check((type + " foo(" + type + " x) { return x << 31; }").c_str(), &settings, "test.cpp", "c++14");
+                check((type + " foo(" + type + " x) { return x << 31; }").c_str(), settings, "test.cpp", Standards::cppstd_t::CPP14);
                 ASSERT_EQUALS("[test.cpp:1]: (portability) Shifting signed 32-bit value by 31 bits is implementation-defined behaviour\n", errout.str());
-                check((type + " f(int x) { return (x = (" + type + ")x << 32); }").c_str(), &settings, "test.cpp", "c++14");
+                check((type + " f(int x) { return (x = (" + type + ")x << 32); }").c_str(), settings, "test.cpp", Standards::cppstd_t::CPP14);
                 ASSERT_EQUALS("[test.cpp:1]: (error) Shifting 32-bit value by 32 bits is undefined behaviour\n", errout.str());
             }
         }
         // 64 bit width types
         {
             // unsigned long long
-            check("unsigned long long foo(unsigned long long x) { return x << 64; }",&settings);
+            check("unsigned long long foo(unsigned long long x) { return x << 64; }",settings);
             ASSERT_EQUALS("[test.cpp:1]: (error) Shifting 64-bit value by 64 bits is undefined behaviour\n", errout.str());
-            check("unsigned long long f(int x) { return (x = (unsigned long long)x << 64); }",&settings);
+            check("unsigned long long f(int x) { return (x = (unsigned long long)x << 64); }",settings);
             ASSERT_EQUALS("[test.cpp:1]: (error) Shifting 64-bit value by 64 bits is undefined behaviour\n", errout.str());
-            check("unsigned long long f(unsigned long long x) { return x << 63; }",&settings);
+            check("unsigned long long f(unsigned long long x) { return x << 63; }",settings);
             ASSERT_EQUALS("", errout.str());
             // [signed] long long
-            check("long long foo(long long x) { return x << 64; }",&settings);
+            check("long long foo(long long x) { return x << 64; }",settings);
             ASSERT_EQUALS("[test.cpp:1]: (error) Shifting 64-bit value by 64 bits is undefined behaviour\n", errout.str());
-            check("long long f(int x) { return (x = (long long)x << 64); }",&settings);
+            check("long long f(int x) { return (x = (long long)x << 64); }",settings);
             ASSERT_EQUALS("[test.cpp:1]: (error) Shifting 64-bit value by 64 bits is undefined behaviour\n", errout.str());
-            check("long long f(long long x) { return x << 63; }",&settings);
+            check("long long f(long long x) { return x << 63; }",settings);
             ASSERT_EQUALS("[test.cpp:1]: (error) Shifting signed 64-bit value by 63 bits is undefined behaviour\n", errout.str());
-            check("long long f(long long x) { return x << 62; }",&settings);
+            check("long long f(long long x) { return x << 62; }",settings);
             ASSERT_EQUALS("", errout.str());
             // signed long long
-            check("signed long long foo(signed long long x) { return x << 64; }",&settings);
+            check("signed long long foo(signed long long x) { return x << 64; }",settings);
             ASSERT_EQUALS("[test.cpp:1]: (error) Shifting 64-bit value by 64 bits is undefined behaviour\n", errout.str());
-            check("signed long long f(long long x) { return (x = (signed long long)x << 64); }",&settings);
+            check("signed long long f(long long x) { return (x = (signed long long)x << 64); }",settings);
             ASSERT_EQUALS("[test.cpp:1]: (error) Shifting 64-bit value by 64 bits is undefined behaviour\n", errout.str());
-            check("signed long long f(signed long long x) { return x << 63; }",&settings);
+            check("signed long long f(signed long long x) { return x << 63; }",settings);
             ASSERT_EQUALS("[test.cpp:1]: (error) Shifting signed 64-bit value by 63 bits is undefined behaviour\n", errout.str());
-            check("signed long long f(signed long long x) { return x << 62; }",&settings);
+            check("signed long long f(signed long long x) { return x << 62; }",settings);
             ASSERT_EQUALS("", errout.str());
 
             // c++14
-            check("signed long long foo(signed long long x) { return x << 64; }",&settings, "test.cpp", "c++14");
+            check("signed long long foo(signed long long x) { return x << 64; }",settings, "test.cpp", Standards::cppstd_t::CPP14);
             ASSERT_EQUALS("[test.cpp:1]: (error) Shifting 64-bit value by 64 bits is undefined behaviour\n", errout.str());
-            check("signed long long f(long long x) { return (x = (signed long long)x << 64); }",&settings, "test.cpp", "c++14");
+            check("signed long long f(long long x) { return (x = (signed long long)x << 64); }",settings, "test.cpp", Standards::cppstd_t::CPP14);
             ASSERT_EQUALS("[test.cpp:1]: (error) Shifting 64-bit value by 64 bits is undefined behaviour\n", errout.str());
-            check("signed long long f(signed long long x) { return x << 63; }",&settings, "test.cpp", "c++14");
+            check("signed long long f(signed long long x) { return x << 63; }",settings, "test.cpp", Standards::cppstd_t::CPP14);
             ASSERT_EQUALS("[test.cpp:1]: (portability) Shifting signed 64-bit value by 63 bits is implementation-defined behaviour\n", errout.str());
-            check("signed long long f(signed long long x) { return x << 62; }",&settings);
+            check("signed long long f(signed long long x) { return x << 62; }",settings);
             ASSERT_EQUALS("", errout.str());
         }
 
-        check("void f() { int x; x = 1 >> 64; }", &settings);
+        check("void f() { int x; x = 1 >> 64; }", settings);
         ASSERT_EQUALS("[test.cpp:1]: (error) Shifting 32-bit value by 64 bits is undefined behaviour\n", errout.str());
 
         check("void foo() {\n"
               "  QList<int> someList;\n"
               "  someList << 300;\n"
-              "}", &settings);
+              "}", settings);
         ASSERT_EQUALS("", errout.str());
 
         // Ticket #6793
@@ -160,14 +154,14 @@ private:
               "const unsigned int f = foo<31>(0);\n"
               "const unsigned int g = foo<100>(0);\n"
               "template<unsigned int I> int hoo(unsigned int x) { return x << 32; }\n"
-              "const unsigned int h = hoo<100>(0);", &settings);
+              "const unsigned int h = hoo<100>(0);", settings);
         ASSERT_EQUALS("[test.cpp:4]: (error) Shifting 32-bit value by 32 bits is undefined behaviour\n"
                       "[test.cpp:1]: (error) Shifting 32-bit value by 100 bits is undefined behaviour\n", errout.str());
 
         // #7266: C++, shift in macro
         check("void f(unsigned int x) {\n"
               "    UINFO(x << 1234);\n"
-              "}");
+              "}", settings0);
         ASSERT_EQUALS("", errout.str());
 
         // #8640
@@ -177,7 +171,7 @@ private:
               "    constexpr const int shift[1] = {32};\n"
               "    constexpr const int ret = a << shift[0];\n" // shift too many bits
               "    return ret;\n"
-              "}");
+              "}", settings0);
         ASSERT_EQUALS("[test.cpp:5]: (error) Shifting 32-bit value by 32 bits is undefined behaviour\n"
                       "[test.cpp:5]: (error) Signed integer overflow for expression 'a<<shift[0]'.\n", errout.str());
 
@@ -188,7 +182,7 @@ private:
               "  if (k > 32)\n"
               "    return 0;\n"
               "  return rm>> k;\n"
-              "}");
+              "}", settings0);
         ASSERT_EQUALS(
             "[test.cpp:4] -> [test.cpp:6]: (warning) Shifting signed 32-bit value by 31 bits is undefined behaviour. See condition at line 4.\n",
             errout.str());
@@ -200,7 +194,7 @@ private:
               "    return 0;\n"
               "  else\n"
               "    return rm>> k;\n"
-              "}");
+              "}", settings0);
         ASSERT_EQUALS(
             "[test.cpp:4] -> [test.cpp:7]: (warning) Shifting signed 32-bit value by 31 bits is undefined behaviour. See condition at line 4.\n",
             errout.str());
@@ -212,20 +206,20 @@ private:
               "    return 0;\n"
               "  else\n"
               "    return rm>> k;\n"
-              "}");
+              "}", settings0);
         ASSERT_EQUALS("", errout.str());
 
         check("static long long f(int x, long long y) {\n"
               "    if (x >= 64)\n"
               "        return 0;\n"
               "    return -(y << (x-1));\n"
-              "}");
+              "}", settings0);
         ASSERT_EQUALS("", errout.str());
 
         check("bool f() {\n"
               "    std::ofstream outfile;\n"
               "    outfile << vec_points[0](0) << static_cast<int>(d) << ' ';\n"
-              "}");
+              "}", settings0);
         ASSERT_EQUALS("", errout.str());
 
         check("void f(unsigned b, int len, unsigned char rem) {\n" // #10773
@@ -236,71 +230,69 @@ private:
               "        if (bits == 512)\n"
               "            len -= 8;\n"
               "    }\n"
-              "}\n");
+              "}\n", settings0);
         ASSERT_EQUALS("", errout.str());
     }
 
     void checkIntegerOverflow() {
-        Settings settings;
-        settings.platform(Settings::Unix32);
-        settings.severity.enable(Severity::warning);
+        const Settings settings = settingsBuilder().severity(Severity::warning).platform(cppcheck::Platform::Type::Unix32).build();
 
-        check("x = (int)0x10000 * (int)0x10000;", &settings);
+        check("x = (int)0x10000 * (int)0x10000;", settings);
         ASSERT_EQUALS("[test.cpp:1]: (error) Signed integer overflow for expression '(int)0x10000*(int)0x10000'.\n", errout.str());
 
-        check("x = (long)0x10000 * (long)0x10000;", &settings);
+        check("x = (long)0x10000 * (long)0x10000;", settings);
         ASSERT_EQUALS("[test.cpp:1]: (error) Signed integer overflow for expression '(long)0x10000*(long)0x10000'.\n", errout.str());
 
         check("void foo() {\n"
               "    int intmax = 0x7fffffff;\n"
               "    return intmax + 1;\n"
-              "}",&settings);
+              "}",settings);
         ASSERT_EQUALS("[test.cpp:3]: (error) Signed integer overflow for expression 'intmax+1'.\n", errout.str());
 
         check("void foo() {\n"
               "    int intmax = 0x7fffffff;\n"
               "    return intmax - 1;\n"
-              "}",&settings);
+              "}",settings);
         ASSERT_EQUALS("", errout.str());
 
         check("int foo(signed int x) {\n"
               "   if (x==123456) {}\n"
               "   return x * x;\n"
-              "}",&settings);
+              "}",settings);
         ASSERT_EQUALS("[test.cpp:2] -> [test.cpp:3]: (warning) Either the condition 'x==123456' is redundant or there is signed integer overflow for expression 'x*x'.\n", errout.str());
 
         check("int foo(signed int x) {\n"
               "   if (x==123456) {}\n"
               "   return -123456 * x;\n"
-              "}",&settings);
+              "}",settings);
         ASSERT_EQUALS("[test.cpp:2] -> [test.cpp:3]: (warning) Either the condition 'x==123456' is redundant or there is signed integer overflow for expression '-123456*x'.\n", errout.str());
 
         check("int foo(signed int x) {\n"
               "   if (x==123456) {}\n"
               "   return 123456U * x;\n"
-              "}",&settings);
+              "}",settings);
         ASSERT_EQUALS("", errout.str());
     }
 
     void signConversion() {
-        Settings settings;
-        settings.platform(Settings::Unix64);
-        check("x = -4 * (unsigned)y;");
+        const Settings settings0;
+        const Settings settings = settingsBuilder().platform(cppcheck::Platform::Type::Unix64).build();
+        check("x = -4 * (unsigned)y;", settings0);
         ASSERT_EQUALS("[test.cpp:1]: (warning) Expression '-4' has a negative value. That is converted to an unsigned value and used in an unsigned calculation.\n", errout.str());
 
-        check("x = (unsigned)y * -4;");
+        check("x = (unsigned)y * -4;", settings0);
         ASSERT_EQUALS("[test.cpp:1]: (warning) Expression '-4' has a negative value. That is converted to an unsigned value and used in an unsigned calculation.\n", errout.str());
 
         check("unsigned int dostuff(int x) {\n" // x is signed
               "  if (x==0) {}\n"
               "  return (x-1)*sizeof(int);\n"
-              "}", &settings);
+              "}", settings);
         ASSERT_EQUALS("[test.cpp:2] -> [test.cpp:3]: (warning) Expression 'x-1' can have a negative value. That is converted to an unsigned value and used in an unsigned calculation.\n", errout.str());
 
         check("unsigned int f1(signed int x, unsigned int y) {" // x is signed
               "  return x * y;\n"
               "}\n"
-              "void f2() { f1(-4,4); }");
+              "void f2() { f1(-4,4); }", settings0);
         ASSERT_EQUALS(
             "[test.cpp:1]: (warning) Expression 'x' can have a negative value. That is converted to an unsigned value and used in an unsigned calculation.\n",
             errout.str());
@@ -308,7 +300,7 @@ private:
         check("unsigned int f1(int x) {"
               "  return x * 5U;\n"
               "}\n"
-              "void f2() { f1(-4); }");
+              "void f2() { f1(-4); }", settings0);
         ASSERT_EQUALS(
             "[test.cpp:1]: (warning) Expression 'x' can have a negative value. That is converted to an unsigned value and used in an unsigned calculation.\n",
             errout.str());
@@ -316,67 +308,64 @@ private:
         check("unsigned int f1(int x) {" // #6168: FP for inner calculation
               "  return 5U * (1234 - x);\n" // <- signed subtraction, x is not sign converted
               "}\n"
-              "void f2() { f1(-4); }");
+              "void f2() { f1(-4); }", settings0);
         ASSERT_EQUALS("", errout.str());
 
         // Don't warn for + and -
         check("void f1(int x) {"
               "  a = x + 5U;\n"
               "}\n"
-              "void f2() { f1(-4); }");
+              "void f2() { f1(-4); }", settings0);
         ASSERT_EQUALS("", errout.str());
 
         check("size_t foo(size_t x) {\n"
               " return -2 * x;\n"
-              "}");
+              "}", settings0);
         ASSERT_EQUALS("[test.cpp:2]: (warning) Expression '-2' has a negative value. That is converted to an unsigned value and used in an unsigned calculation.\n", errout.str());
     }
 
     void longCastAssign() {
-        Settings settings;
-        settings.severity.enable(Severity::style);
-        settings.platform(Settings::Unix64);
+        const Settings settings = settingsBuilder().severity(Severity::style).platform(cppcheck::Platform::Type::Unix64).build();
 
         check("long f(int x, int y) {\n"
               "  const long ret = x * y;\n"
               "  return ret;\n"
-              "}\n", &settings);
+              "}\n", settings);
         ASSERT_EQUALS("[test.cpp:2]: (style) int result is assigned to long variable. If the variable is long to avoid loss of information, then you have loss of information.\n", errout.str());
 
         check("long f() {\n"
               "  const long long ret = 256 * (1 << 10);\n"
               "  return ret;\n"
-              "}\n", &settings);
+              "}\n", settings);
         ASSERT_EQUALS("", errout.str());
 
         // typedef
         check("long f(int x, int y) {\n"
               "  const size_t ret = x * y;\n"
               "  return ret;\n"
-              "}\n", &settings);
+              "}\n", settings);
         ASSERT_EQUALS("", errout.str());
 
         // astIsIntResult
         check("long f(int x, int y) {\n"
               "  const long ret = (long)x * y;\n"
               "  return ret;\n"
-              "}\n", &settings);
+              "}\n", settings);
         ASSERT_EQUALS("", errout.str());
     }
 
     void longCastReturn() {
-        Settings settings;
-        settings.severity.enable(Severity::style);
+        const Settings settings = settingsBuilder().severity(Severity::style).build();
 
         check("long f(int x, int y) {\n"
               "  return x * y;\n"
-              "}\n", &settings);
+              "}\n", settings);
         ASSERT_EQUALS("[test.cpp:2]: (style) int result is returned as long value. If the return value is long to avoid loss of information, then you have loss of information.\n", errout.str());
 
         // typedef
         check("size_t f(int x, int y) {\n"
               "  return x * y;\n"
-              "}\n", &settings);
+              "}\n", settings);
         ASSERT_EQUALS("", errout.str());
     }
 
@@ -391,42 +380,43 @@ private:
     }
 
     void checkFloatToIntegerOverflow() {
-        check("x = (int)1E100;");
+        const Settings settings;
+        check("x = (int)1E100;", settings);
         ASSERT_EQUALS("[test.cpp:1]: (error) Undefined behaviour: float () to integer conversion overflow.\n", removeFloat(errout.str()));
 
         check("void f(void) {\n"
               "  return (int)1E100;\n"
-              "}");
+              "}", settings);
         ASSERT_EQUALS("[test.cpp:2]: (error) Undefined behaviour: float () to integer conversion overflow.\n", removeFloat(errout.str()));
 
         check("void f(void) {\n"
               "  return (int)-1E100;\n"
-              "}");
+              "}", settings);
         ASSERT_EQUALS("[test.cpp:2]: (error) Undefined behaviour: float () to integer conversion overflow.\n", removeFloat(errout.str()));
 
         check("void f(void) {\n"
               "  return (short)1E6;\n"
-              "}");
+              "}", settings);
         ASSERT_EQUALS("[test.cpp:2]: (error) Undefined behaviour: float () to integer conversion overflow.\n", removeFloat(errout.str()));
 
         check("void f(void) {\n"
               "  return (unsigned char)256.0;\n"
-              "}");
+              "}", settings);
         ASSERT_EQUALS("[test.cpp:2]: (error) Undefined behaviour: float () to integer conversion overflow.\n", removeFloat(errout.str()));
 
         check("void f(void) {\n"
               "  return (unsigned char)255.5;\n"
-              "}");
+              "}", settings);
         ASSERT_EQUALS("", removeFloat(errout.str()));
 
         check("void f(void) {\n"
               "  char c = 1234.5;\n"
-              "}");
+              "}", settings);
         ASSERT_EQUALS("[test.cpp:2]: (error) Undefined behaviour: float () to integer conversion overflow.\n", removeFloat(errout.str()));
 
         check("char f(void) {\n"
               "  return 1234.5;\n"
-              "}");
+              "}", settings);
         ASSERT_EQUALS("[test.cpp:2]: (error) Undefined behaviour: float () to integer conversion overflow.\n", removeFloat(errout.str()));
     }
 };
