@@ -20,12 +20,18 @@
 
 #include <algorithm>
 #include <array>
+#include <cstdlib>
 #include <fstream> // IWYU pragma: keep
 #include <functional>
 #include <iostream>
+#include <map>
+#include <set>
 #include <string>
+#include <utility>
 #include <vector>
 
+#include "config.h"
+
 #include "../cli/filelister.h"
 #include "../lib/pathmatch.h"
 #include "../lib/utils.h"
@@ -163,7 +169,7 @@ static int write_vcxproj(const std::string &proj_name, const std::function<void(
     std::string outstr;
 
     {
-        // treat as binary to prevent inplicit line ending conversions
+        // treat as binary to prevent implicit line ending conversions
         std::ifstream in(proj_name, std::ios::binary);
         if (!in.is_open()) {
             std::cerr << "Could not open " << proj_name << std::endl;
@@ -208,19 +214,19 @@ static int write_vcxproj(const std::string &proj_name, const std::function<void(
             outstr.resize(pos+1, '\0');
     }
 
-    // treat as binary to prevent inplicit line ending conversions
+    // treat as binary to prevent implicit line ending conversions
     std::ofstream out(proj_name, std::ios::binary|std::ios::trunc);
     out << outstr;
 
     return EXIT_SUCCESS;
 }
 
-enum ClType { Compile, Include, Precompile, PrecompileNoPCRE };
+enum ClType { Compile, Include, Precompile };
 
 static std::string make_vcxproj_cl_entry(const std::string& file, ClType type)
 {
     std::string outstr;
-    if (type == Precompile || type == PrecompileNoPCRE) {
+    if (type == Precompile) {
         outstr += R"(    <ClCompile Include=")";
         outstr += file;
         outstr += R"(">)";
@@ -229,22 +235,18 @@ static std::string make_vcxproj_cl_entry(const std::string& file, ClType type)
         outstr += "\r\n";
         outstr += R"(      <PrecompiledHeader Condition="'$(Configuration)|$(Platform)'=='Debug|Win32'">Create</PrecompiledHeader>)";
         outstr += "\r\n";
-        if (type == Precompile) {
-            outstr += R"(      <PrecompiledHeader Condition="'$(Configuration)|$(Platform)'=='Release-PCRE|Win32'">Create</PrecompiledHeader>)";
-            outstr += "\r\n";
-            outstr += R"(      <PrecompiledHeader Condition="'$(Configuration)|$(Platform)'=='Debug-PCRE|Win32'">Create</PrecompiledHeader>)";
-            outstr += "\r\n";
-        }
+        outstr += R"(      <PrecompiledHeader Condition="'$(Configuration)|$(Platform)'=='Release-PCRE|Win32'">Create</PrecompiledHeader>)";
+        outstr += "\r\n";
+        outstr += R"(      <PrecompiledHeader Condition="'$(Configuration)|$(Platform)'=='Debug-PCRE|Win32'">Create</PrecompiledHeader>)";
+        outstr += "\r\n";
         outstr += R"(      <PrecompiledHeader Condition="'$(Configuration)|$(Platform)'=='Release|Win32'">Create</PrecompiledHeader>)";
         outstr += "\r\n";
         outstr += R"(      <PrecompiledHeader Condition="'$(Configuration)|$(Platform)'=='Debug|x64'">Create</PrecompiledHeader>)";
         outstr += "\r\n";
-        if (type == Precompile) {
-            outstr += R"(      <PrecompiledHeader Condition="'$(Configuration)|$(Platform)'=='Release-PCRE|x64'">Create</PrecompiledHeader>)";
-            outstr += "\r\n";
-            outstr += R"(      <PrecompiledHeader Condition="'$(Configuration)|$(Platform)'=='Debug-PCRE|x64'">Create</PrecompiledHeader>)";
-            outstr += "\r\n";
-        }
+        outstr += R"(      <PrecompiledHeader Condition="'$(Configuration)|$(Platform)'=='Release-PCRE|x64'">Create</PrecompiledHeader>)";
+        outstr += "\r\n";
+        outstr += R"(      <PrecompiledHeader Condition="'$(Configuration)|$(Platform)'=='Debug-PCRE|x64'">Create</PrecompiledHeader>)";
+        outstr += "\r\n";
         outstr += "    </ClCompile>\r\n";
         return outstr;
     }
@@ -341,7 +343,8 @@ int main(int argc, char **argv)
     // Visual Studio projects
     write_vcxproj("cli/cli.vcxproj", [&](std::string &outstr){
         for (const std::string &clifile: clifiles) {
-            outstr += make_vcxproj_cl_entry(clifile.substr(4), Compile);
+            const std::string c = clifile.substr(4);
+            outstr += make_vcxproj_cl_entry(c, c == "executor.cpp" ? Precompile : Compile);
         }
     }, [&](std::string &outstr){
         for (const std::string &clifile_h: clifiles_h) {
@@ -376,7 +379,7 @@ int main(int argc, char **argv)
 
         for (const std::string &testfile: testfiles) {
             const std::string t = testfile.substr(5);
-            outstr += make_vcxproj_cl_entry(t, t == "fixture.cpp" ? PrecompileNoPCRE : Compile);
+            outstr += make_vcxproj_cl_entry(t, t == "fixture.cpp" ? Precompile : Compile);
         }
     }, [&](std::string &outstr){
         for (const std::string &clifile_h: clifiles_h) {
@@ -652,7 +655,8 @@ int main(int argc, char **argv)
     fout << "cppcheck: $(LIBOBJ) $(CLIOBJ) $(EXTOBJ)\n";
     fout << "\t$(CXX) $(CPPFLAGS) $(CXXFLAGS) -o $@ $^ $(LIBS) $(LDFLAGS) $(RDYNAMIC)\n\n";
     fout << "all:\tcppcheck testrunner\n\n";
-    fout << "testrunner: $(TESTOBJ) $(LIBOBJ) $(EXTOBJ) cli/executor.o cli/processexecutor.o cli/threadexecutor.o cli/cmdlineparser.o cli/cppcheckexecutor.o cli/cppcheckexecutorseh.o cli/cppcheckexecutorsig.o cli/stacktrace.o cli/filelister.o\n";
+    // TODO: generate from clifiles
+    fout << "testrunner: $(TESTOBJ) $(LIBOBJ) $(EXTOBJ) cli/executor.o cli/processexecutor.o cli/singleexecutor.o cli/threadexecutor.o cli/cmdlineparser.o cli/cppcheckexecutor.o cli/cppcheckexecutorseh.o cli/cppcheckexecutorsig.o cli/stacktrace.o cli/filelister.o\n";
     fout << "\t$(CXX) $(CPPFLAGS) $(CXXFLAGS) -o $@ $^ $(LIBS) $(LDFLAGS) $(RDYNAMIC)\n\n";
     fout << "test:\tall\n";
     fout << "\t./testrunner\n\n";
